==> symbol-processing-aa-embeddable-2.1.0-1.0.29-sources.jar: org/jetbrains/kotlin/light/classes/symbol/parameters/SymbolLightParameterList.kt <==
/*
 * Copyright 2010-2023 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.light.classes.symbol.parameters

import ksp.com.intellij.psi.JavaElementVisitor
import ksp.com.intellij.psi.PsiElementVisitor
import ksp.com.intellij.psi.PsiParameter
import ksp.com.intellij.psi.PsiParameterList
import ksp.com.intellij.psi.impl.light.LightParameterListBuilder
import ksp.org.jetbrains.kotlin.analysis.api.symbols.KaCallableSymbol
import ksp.org.jetbrains.kotlin.analysis.api.symbols.pointers.KaSymbolPointer
import ksp.org.jetbrains.kotlin.asJava.classes.lazyPub
import ksp.org.jetbrains.kotlin.asJava.elements.KtLightElement
import ksp.org.jetbrains.kotlin.asJava.elements.KtLightElementBase
import ksp.org.jetbrains.kotlin.light.classes.symbol.methods.SymbolLightMethodBase
import ksp.org.jetbrains.kotlin.psi.KtFunction
import ksp.org.jetbrains.kotlin.psi.KtParameterList

internal class SymbolLightParameterList(
    private val parent: SymbolLightMethodBase,
    private val callableWithReceiverSymbolPointer: KaSymbolPointer<KaCallableSymbol>? = null,
    parameterPopulator: (LightParameterListBuilder) -> Unit = {},
) : KtLightElement<KtParameterList, PsiParameterList>,
    // With this, a parent chain is properly built: from SymbolLightParameter through SymbolLightParameterList to SymbolLightMethod
    KtLightElementBase(parent),
    // NB: we can't use delegation here, which will conflict getTextRange from KtLightElementBase
    PsiParameterList {

    override val kotlinOrigin: KtParameterList?
        get() = (parent.kotlinOrigin as? KtFunction)?.valueParameterList

    private val clsDelegate: PsiParameterList by lazyPub {
        val builder = LightParameterListBuilder(manager, language)

        callableWithReceiverSymbolPointer?.let {
            SymbolLightParameterForReceiver.tryGet(it, parent)?.let { receiver ->
                builder.addParameter(receiver)
            }
        }

        parameterPopulator.invoke(builder)

        builder
    }

    override fun accept(visitor: PsiElementVisitor) {
        if (visitor is JavaElementVisitor) {
            visitor.visitParameterList(this)
        } else {
            visitor.visitElement(this)
        }
    }

    override fun getParameter(index: Int): PsiParameter? = clsDelegate.getParameter(index)
    override fun getParameters(): Array<PsiParameter> = clsDelegate.parameters
    override fun getParameterIndex(p: PsiParameter): Int = clsDelegate.getParameterIndex(p)
    override fun getParametersCount(): Int = clsDelegate.parametersCount
}

==> symbol-processing-aa-embeddable-2.2.0-2.0.2-sources.jar: org/jetbrains/kotlin/light/classes/symbol/parameters/SymbolLightParameterList.kt <==
/*
 * Copyright 2010-2025 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.light.classes.symbol.parameters

import ksp.com.intellij.psi.JavaElementVisitor
import ksp.com.intellij.psi.PsiElementVisitor
import ksp.com.intellij.psi.PsiParameter
import ksp.com.intellij.psi.PsiParameterList
import ksp.com.intellij.psi.impl.light.LightParameterListBuilder
import ksp.org.jetbrains.kotlin.analysis.api.symbols.KaCallableSymbol
import ksp.org.jetbrains.kotlin.analysis.api.symbols.contextParameters
import ksp.org.jetbrains.kotlin.analysis.api.symbols.pointers.KaSymbolPointer
import ksp.org.jetbrains.kotlin.asJava.classes.lazyPub
import ksp.org.jetbrains.kotlin.asJava.elements.KtLightElement
import ksp.org.jetbrains.kotlin.asJava.elements.KtLightElementBase
import ksp.org.jetbrains.kotlin.light.classes.symbol.methods.SymbolLightMethodBase
import ksp.org.jetbrains.kotlin.light.classes.symbol.withSymbol
import ksp.org.jetbrains.kotlin.psi.KtFunction
import ksp.org.jetbrains.kotlin.psi.KtParameterList

internal class SymbolLightParameterList(
    private val parent: SymbolLightMethodBase,
    private val correspondingCallablePointer: KaSymbolPointer<KaCallableSymbol>? = null,
    parameterPopulator: (LightParameterListBuilder) -> Unit = {},
) : KtLightElement<KtParameterList, PsiParameterList>,
    // With this, a parent chain is properly built: from SymbolLightParameter through SymbolLightParameterList to SymbolLightMethod
    KtLightElementBase(parent),
    // NB: we can't use delegation here, which will conflict getTextRange from KtLightElementBase
    PsiParameterList {

    override val kotlinOrigin: KtParameterList?
        get() = (parent.kotlinOrigin as? KtFunction)?.valueParameterList

    private val clsDelegate: PsiParameterList by lazyPub {
        val builder = LightParameterListBuilder(manager, language)

        correspondingCallablePointer?.withSymbol(parent.ktModule) { callable ->
            for (parameterSymbol in callable.contextParameters) {
                builder.addParameter(SymbolLightContextParameter(this, parameterSymbol, parent))
            }
        }

        correspondingCallablePointer?.let {
            SymbolLightParameterForReceiver.tryGet(it, parent)?.let { receiver ->
                builder.addParameter(receiver)
            }
        }

        parameterPopulator.invoke(builder)

        builder
    }

    override fun accept(visitor: PsiElementVisitor) {
        if (visitor is JavaElementVisitor) {
            visitor.visitParameterList(this)
        } else {
            visitor.visitElement(this)
        }
    }

    override fun getParameter(index: Int): PsiParameter? = clsDelegate.getParameter(index)
    override fun getParameters(): Array<PsiParameter> = clsDelegate.parameters
    override fun getParameterIndex(p: PsiParameter): Int = clsDelegate.getParameterIndex(p)
    override fun getParametersCount(): Int = clsDelegate.parametersCount
}
